@@ -26,9 +26,6 @@
 # include <sys/types.h>
 # include <unistd.h>
 #endif
-#ifdef CHROMIUM_THREADSAFE
-#include "cr_threads.h"
-#endif
 
 #ifdef VBOX_WITH_WDDM
 #include <d3d9types.h>
@@ -55,10 +52,22 @@ static char* gsViewportHackApps[] = {"googleearth.exe", NULL};
 #endif
 
 static int stub_initialized = 0;
+#ifdef WINDOWS
+static CRmutex stub_init_mutex;
+#define STUB_INIT_LOCK() do { crLockMutex(&stub_init_mutex); } while (0)
+#define STUB_INIT_UNLOCK() do { crUnlockMutex(&stub_init_mutex); } while (0)
+#else
+#define STUB_INIT_LOCK() do { } while (0)
+#define STUB_INIT_UNLOCK() do { } while (0)
+#endif
 
 /* NOTE: 'SPUDispatchTable glim' is declared in NULLfuncs.py now */
 /* NOTE: 'SPUDispatchTable stubThreadsafeDispatch' is declared in tsfuncs.c */
 Stub stub;
+#ifdef CHROMIUM_THREADSAFE
+static bool g_stubIsCurrentContextTSDInited;
+CRtsd g_stubCurrentContextTSD;
+#endif
 
 
 static void stubInitNativeDispatch( void )
@@ -102,7 +111,7 @@ static void stubCheckWindowState(WindowInfo *window, GLboolean bFlushOnChange)
 
         devMode.dmSize = sizeof(DEVMODE);
         EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &devMode);
-        
+
         if (devMode.dmPelsWidth!=window->dmPelsWidth || devMode.dmPelsHeight!=window->dmPelsHeight)
         {
             crDebug("Resolution changed(%d,%d), forcing window Pos/Size update", devMode.dmPelsWidth, devMode.dmPelsHeight);
@@ -192,9 +201,11 @@ static void stubCheckWindowsCB(unsigned long key, void *data1, void *data2)
 
 static void stubCheckWindowsState(void)
 {
+    ContextInfo *context = stubGetCurrentContext();
+
     CRASSERT(stub.trackWindowSize || stub.trackWindowPos);
 
-    if (!stub.currentContext)
+    if (!context)
         return;
 
 #if defined(WINDOWS) && defined(VBOX_WITH_WDDM)
@@ -206,8 +217,8 @@ static void stubCheckWindowsState(void)
     crLockMutex(&stub.mutex);
 #endif
 
-    stubCheckWindowState(stub.currentContext->currentDrawable, GL_TRUE);
-    crHashtableWalk(stub.windowTable, stubCheckWindowsCB, stub.currentContext);
+    stubCheckWindowState(context->currentDrawable, GL_TRUE);
+    crHashtableWalk(stub.windowTable, stubCheckWindowsCB, context);
 
 #if defined(CR_NEWWINTRACK) && !defined(WINDOWS)
     crUnlockMutex(&stub.mutex);
@@ -241,10 +252,11 @@ static void SPU_APIENTRY trapViewport(GLint x, GLint y, GLsizei w, GLsizei h)
     }
     else
     {
+        ContextInfo *context = stubGetCurrentContext();
         int winX, winY;
         unsigned int winW, winH;
         WindowInfo *pWindow;
-        pWindow = stub.currentContext->currentDrawable;
+        pWindow = context->currentDrawable;
         stubGetWindowGeometry(pWindow, &winX, &winY, &winW, &winH);
         origViewport(0, 0, winW, winH);
     }
@@ -267,7 +279,8 @@ static void SPU_APIENTRY trapScissor(GLint x, GLint y, GLsizei w, GLsizei h)
     int winX, winY;
     unsigned int winW, winH;
     WindowInfo *pWindow;
-    pWindow = stub.currentContext->currentDrawable;
+    ContextInfo *context = stubGetCurrentContext();
+    pWindow = context->currentDrawable;
     stubGetWindowGeometry(pWindow, &winX, &winY, &winW, &winH);
     origScissor(0, 0, winW, winH);
 }
@@ -310,12 +323,9 @@ static void hsWalkStubDestroyContexts(unsigned long key, void *data1, void *data
  * This is called when we exit.
  * We call all the SPU's cleanup functions.
  */
-static void stubSPUTearDown(void)
+static void stubSPUTearDownLocked(void)
 {
-    crDebug("stubSPUTearDown");
-    if (!stub_initialized) return;
-
-    stub_initialized = 0;
+    crDebug("stubSPUTearDownLocked");
 
 #ifdef WINDOWS
 # ifndef CR_NEWWINTRACK
@@ -326,10 +336,15 @@ static void stubSPUTearDown(void)
 #ifdef CR_NEWWINTRACK
     ASMAtomicWriteBool(&stub.bShutdownSyncThread, true);
 #endif
-  
+
     //delete all created contexts
     stubMakeCurrent( NULL, NULL);
+
+    /* the lock order is windowTable->contextTable (see wglMakeCurrent_prox, glXMakeCurrent)
+     * this is why we need to take a windowTable lock since we will later do stub.windowTable access & locking */
+    crHashtableLock(stub.windowTable);
     crHashtableWalk(stub.contextTable, hsWalkStubDestroyContexts, NULL);
+    crHashtableUnlock(stub.windowTable);
 
     /* shutdown, now trap any calls to a NULL dispatcher */
     crSPUCopyDispatchTable(&glim, &stubNULLDispatch);
@@ -358,6 +373,22 @@ static void stubSPUTearDown(void)
     crFreeHashtable(stub.contextTable, NULL);
 
     crMemset(&stub, 0, sizeof(stub));
+
+}
+
+/**
+ * This is called when we exit.
+ * We call all the SPU's cleanup functions.
+ */
+static void stubSPUTearDown(void)
+{
+    STUB_INIT_LOCK();
+    if (stub_initialized)
+    {
+        stubSPUTearDownLocked();
+        stub_initialized = 0;
+    }
+    STUB_INIT_UNLOCK();
 }
 
 static void stubSPUSafeTearDown(void)
@@ -408,7 +439,7 @@ static void stubSPUSafeTearDown(void)
 
             /*Same issue as on linux, RTThreadWait exits before system thread is terminated, which leads
              * to issues as our dll goes to be unloaded.
-             *@todo 
+             *@todo
              *We usually call this function from DllMain which seems to be holding some lock and thus we have to
              * kill thread via TerminateThread.
              */
@@ -482,6 +513,16 @@ static void stubSignalHandler(int signo)
     exit(0);  /* this causes stubExitHandler() to be called */
 }
 
+#ifndef RT_OS_WINDOWS
+# ifdef CHROMIUM_THREADSAFE
+static DECLCALLBACK(void) stubThreadTlsDtor(void *pvValue)
+{
+    ContextInfo *pCtx = (ContextInfo*)pvValue;
+    VBoxTlsRefRelease(pCtx);
+}
+# endif
+#endif
+
 
 /**
  * Init variables in the stub structure, install signal handler.
@@ -516,7 +557,16 @@ static void stubInitVars(void)
 
     stub.freeContextNumber = MAGIC_CONTEXT_BASE;
     stub.contextTable = crAllocHashtable();
-    stub.currentContext = NULL;
+#ifndef RT_OS_WINDOWS
+# ifdef CHROMIUM_THREADSAFE
+    if (!g_stubIsCurrentContextTSDInited)
+    {
+        crInitTSDF(&g_stubCurrentContextTSD, stubThreadTlsDtor);
+        g_stubIsCurrentContextTSDInited = true;
+    }
+# endif
+#endif
+    stubSetCurrentContext(NULL);
 
     stub.windowTable = crAllocHashtable();
 
@@ -652,7 +702,7 @@ LookupMothershipConfig(const char *procName)
         fgets(line, 999, f);
         line[crStrlen(line) - 1] = 0; /* remove trailing newline */
         if (crStrncmp(line, procName, procNameLen) == 0 &&
-            (line[procNameLen] == ' ' || line[procNameLen] == '\t')) 
+            (line[procNameLen] == ' ' || line[procNameLen] == '\t'))
         {
             crWarning("Using Chromium configuration for %s from %s",
                                 procName, configPath);
@@ -887,7 +937,7 @@ static void stubSyncTrUpdateWindowCB(unsigned long key, void *data1, void *data2
 
         if (pWindow->hVisibleRegion!=INVALID_HANDLE_VALUE)
         {
-            CombineRgn(hNewRgn, pWindow->hVisibleRegion, hNewRgn, 
+            CombineRgn(hNewRgn, pWindow->hVisibleRegion, hNewRgn,
                        pRegions->pRegions->fFlags.bAddHiddenRects ? RGN_DIFF:RGN_OR);
 
             if (!EqualRgn(pWindow->hVisibleRegion, hNewRgn))
@@ -1100,16 +1150,16 @@ static DECLCALLBACK(int) stubSyncThreadProc(RTTHREAD ThreadSelf, void *pvUser)
  * Do one-time initializations for the faker.
  * Returns TRUE on success, FALSE otherwise.
  */
-bool
-stubInit(void)
+static bool
+stubInitLocked(void)
 {
     /* Here is where we contact the mothership to find out what we're supposed
-     * to  be doing.  Networking code in a DLL initializer.  I sure hope this 
-     * works :) 
-     * 
+     * to  be doing.  Networking code in a DLL initializer.  I sure hope this
+     * works :)
+     *
      * HOW can I pass the mothership address to this if I already know it?
      */
-    
+
     CRConnection *conn = NULL;
     char response[1024];
     char **spuchain;
@@ -1120,9 +1170,6 @@ stubInit(void)
     int i;
     int disable_sync = 0;
 
-    if (stub_initialized)
-        return true;
-
     stubInitVars();
 
     crGetProcName(response, 1024);
@@ -1201,7 +1248,7 @@ stubInit(void)
 
     crSPUInitDispatchTable( &glim );
 
-    /* This is unlikely to change -- We still want to initialize our dispatch 
+    /* This is unlikely to change -- We still want to initialize our dispatch
      * table with the functions of the first SPU in the chain. */
     stubInitSPUDispatch( stub.spu );
 
@@ -1255,15 +1302,32 @@ raise(SIGINT);*/
     stub.bHaveXFixes = GL_FALSE;
 #endif
 
-    stub_initialized = 1;
     return true;
 }
 
-/* Sigh -- we can't do initialization at load time, since Windows forbids 
+/**
+ * Do one-time initializations for the faker.
+ * Returns TRUE on success, FALSE otherwise.
+ */
+bool
+stubInit(void)
+{
+    bool bRc = true;
+    /* we need to serialize the initialization, otherwise racing is possible
+     * for XPDM-based d3d when a d3d switcher is testing the gl lib in two or more threads
+     * NOTE: the STUB_INIT_LOCK/UNLOCK is a NOP for non-win currently */
+    STUB_INIT_LOCK();
+    if (!stub_initialized)
+        bRc = stub_initialized = stubInitLocked();
+    STUB_INIT_UNLOCK();
+    return bRc;
+}
+
+/* Sigh -- we can't do initialization at load time, since Windows forbids
  * the loading of other libraries from DLLMain. */
 
 #ifdef LINUX
-/* GCC crap 
+/* GCC crap
  *void (*stub_init_ptr)(void) __attribute__((section(".ctors"))) = __stubInit; */
 #endif
 
@@ -1271,17 +1335,76 @@ raise(SIGINT);*/
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 
+#ifdef DEBUG_misha
+ /* debugging: this is to be able to catch first-chance notifications
+  * for exceptions other than EXCEPTION_BREAKPOINT in kernel debugger */
+# define VDBG_VEHANDLER
+#endif
+
+#ifdef VDBG_VEHANDLER
+static PVOID g_VBoxWDbgVEHandler = NULL;
+LONG WINAPI vboxVDbgVectoredHandler(struct _EXCEPTION_POINTERS *pExceptionInfo)
+{
+    PEXCEPTION_RECORD pExceptionRecord = pExceptionInfo->ExceptionRecord;
+    PCONTEXT pContextRecord = pExceptionInfo->ContextRecord;
+    switch (pExceptionRecord->ExceptionCode)
+    {
+        case EXCEPTION_BREAKPOINT:
+        case EXCEPTION_ACCESS_VIOLATION:
+        case EXCEPTION_STACK_OVERFLOW:
+        case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:
+        case EXCEPTION_FLT_DIVIDE_BY_ZERO:
+        case EXCEPTION_FLT_INVALID_OPERATION:
+        case EXCEPTION_INT_DIVIDE_BY_ZERO:
+        case EXCEPTION_ILLEGAL_INSTRUCTION:
+            CRASSERT(0);
+            break;
+        default:
+            break;
+    }
+    return EXCEPTION_CONTINUE_SEARCH;
+}
+
+void vboxVDbgVEHandlerRegister()
+{
+    CRASSERT(!g_VBoxWDbgVEHandler);
+    g_VBoxWDbgVEHandler = AddVectoredExceptionHandler(1,vboxVDbgVectoredHandler);
+    CRASSERT(g_VBoxWDbgVEHandler);
+}
+
+void vboxVDbgVEHandlerUnregister()
+{
+    ULONG uResult;
+    if (g_VBoxWDbgVEHandler)
+    {
+        uResult = RemoveVectoredExceptionHandler(g_VBoxWDbgVEHandler);
+        CRASSERT(uResult);
+        g_VBoxWDbgVEHandler = NULL;
+    }
+}
+#endif
+
 /* Windows crap */
 BOOL WINAPI DllMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
 {
     (void) lpvReserved;
 
-    switch (fdwReason) 
+    switch (fdwReason)
     {
     case DLL_PROCESS_ATTACH:
     {
         CRNetServer ns;
 
+#ifdef CHROMIUM_THREADSAFE
+        crInitTSD(&g_stubCurrentContextTSD);
+#endif
+
+        crInitMutex(&stub_init_mutex);
+
+#ifdef VDBG_VEHANDLER
+        vboxVDbgVEHandlerRegister();
+#endif
+
         crNetInit(NULL, NULL);
         ns.name = "vboxhgcm://host:0";
         ns.buffer_size = 1024;
@@ -1289,6 +1412,9 @@ BOOL WINAPI DllMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
         if (!ns.conn)
         {
             crDebug("Failed to connect to host (is guest 3d acceleration enabled?), aborting ICD load.");
+#ifdef VDBG_VEHANDLER
+            vboxVDbgVEHandlerUnregister();
+#endif
             return FALSE;
         }
         else
@@ -1300,30 +1426,41 @@ BOOL WINAPI DllMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
     case DLL_PROCESS_DETACH:
     {
         stubSPUSafeTearDown();
+
+#ifdef CHROMIUM_THREADSAFE
+        crFreeTSD(&g_stubCurrentContextTSD);
+#endif
+
+#ifdef VDBG_VEHANDLER
+        vboxVDbgVEHandlerUnregister();
+#endif
         break;
     }
 
-#if 0
     case DLL_THREAD_ATTACH:
     {
+#if 0
         if (stub_initialized)
         {
             CRASSERT(stub.spu);
             stub.spu->dispatch_table.VBoxPackAttachThread();
         }
+#endif
         break;
     }
 
     case DLL_THREAD_DETACH:
     {
+        stubSetCurrentContext(NULL);
+#if 0
         if (stub_initialized)
         {
             CRASSERT(stub.spu);
             stub.spu->dispatch_table.VBoxPackDetachThread();
         }
+#endif
         break;
     }
-#endif
 
     default:
         break;
