@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# Linux Additions X11 setup init script ($Revision: 38093 $)
+# Linux Additions X11 setup init script ($Revision: 74559 $)
 #
 
 #
@@ -283,11 +283,11 @@ setup()
     dox11config="true"
     # By default, we want to run our xorg.conf setup script
     setupxorgconf="true"
-    # But not to install the configuration file into xorg.conf.d
-    doxorgconfd=""
-    # But without the workaround for SUSE 11.1 not doing input auto-detection
-    newmouse=""
-    # By default we want to use hal/udev/whatever for auto-loading the mouse driver
+    # On all but the oldest X servers we want to use our new mouse
+    # driver.
+    newmouse="--newMouse"
+    # On more recent servers our kernel mouse driver will be used
+    # automatically
     automouse="--autoMouse"
     # We need to tell our xorg.conf hacking script whether /dev/psaux exists
     nopsaux="--nopsaux"
@@ -296,6 +296,8 @@ setup()
     vboxvideo_src=
     # The mouse driver to install for X.Org 6.9+
     vboxmouse_src=
+    # The driver extension
+    driver_ext=".so"
 
     modules_dir=`X -showDefaultModulePath 2>&1` || modules_dir=
     if [ -z "$modules_dir" ]; then
@@ -324,96 +326,91 @@ setup()
             echo "installing the X.Org drivers."
             dox11config=""
             ;;
+        1.11.* )
+            xserver_version="X.Org Server 1.11"
+            vboxvideo_src=vboxvideo_drv_111.so
+            # Does Fedora still ship without vboxvideo detection?
+            # test "$system" = "redhat" || setupxorgconf=""
+            ;;
         1.10.* )
-            begin "Installing X.Org Server 1.10 modules"
+            xserver_version="X.Org Server 1.10"
             vboxvideo_src=vboxvideo_drv_110.so
-            vboxmouse_src=vboxmouse_drv_110.so
-            doxorgconfd="true"
             # Does Fedora still ship without vboxvideo detection?
             # test "$system" = "redhat" || setupxorgconf=""
             ;;
         1.9.* )
-            begin "Installing X.Org Server 1.9 modules"
+            xserver_version="X.Org Server 1.9"
             vboxvideo_src=vboxvideo_drv_19.so
-            vboxmouse_src=vboxmouse_drv_19.so
-            doxorgconfd="true"
             # Fedora 14 looks likely to ship without vboxvideo detection
             # test "$system" = "redhat" || setupxorgconf=""
             ;;
         1.8.* )
-            begin "Installing X.Org Server 1.8 modules"
+            xserver_version="X.Org Server 1.8"
             vboxvideo_src=vboxvideo_drv_18.so
-            vboxmouse_src=vboxmouse_drv_18.so
-            doxorgconfd="true"
             # Fedora 13 shipped without vboxvideo detection
             test "$system" = "redhat" || setupxorgconf=""
             ;;
         1.7.* )
-            begin "Installing X.Org Server 1.7 modules"
+            xserver_version="X.Org Server 1.7"
             vboxvideo_src=vboxvideo_drv_17.so
-            vboxmouse_src=vboxmouse_drv_17.so
             setupxorgconf=""
-            test "$system" = "debian" && doxorgconfd="true"
             ;;
         1.6.* )
-            begin "Installing X.Org Server 1.6 modules"
+            xserver_version="X.Org Server 1.6"
             vboxvideo_src=vboxvideo_drv_16.so
             vboxmouse_src=vboxmouse_drv_16.so
             # SUSE SLE* with X.Org 1.6 does not do input autodetection;
             # openSUSE does.
             if grep -q -E '^SLE[^ ]' /etc/SuSE-brand 2>/dev/null; then
                 automouse=""
-                newmouse="--newMouse"
             else
                 test "$system" = "suse" && setupxorgconf=""
             fi
             ;;
         1.5.* )
-            begin "Installing X.Org Server 1.5 modules"
+            xserver_version="X.Org Server 1.5"
             vboxvideo_src=vboxvideo_drv_15.so
             vboxmouse_src=vboxmouse_drv_15.so
             # SUSE with X.Org 1.5 is another special case, and is also
             # handled specially
-            test "$system" = "suse" &&
-            { automouse=""; newmouse="--newMouse"; }
+            test "$system" = "suse" && automouse=""
             ;;
         1.4.* )
-            begin "Installing X.Org Server 1.4 modules"
+            xserver_version="X.Org Server 1.4"
             vboxvideo_src=vboxvideo_drv_14.so
             vboxmouse_src=vboxmouse_drv_14.so
             automouse=""
-            newmouse="--newMouse"
             ;;
         1.3.* )
             # This was the first release which gave the server version number
             # rather than the X11 release version when you did 'X -version'.
-            begin "Installing X.Org Server 1.3 modules"
+            xserver_version="X.Org Server 1.3"
             vboxvideo_src=vboxvideo_drv_13.so
             vboxmouse_src=vboxmouse_drv_13.so
             automouse=""
-            newmouse="--newMouse"
             ;;
         7.1.* | 7.2.* )
-            begin "Installing X.Org 7.1 modules"
+            xserver_version="X.Org 7.1"
             vboxvideo_src=vboxvideo_drv_71.so
             vboxmouse_src=vboxmouse_drv_71.so
             automouse=""
+            newmouse=""
             ;;
         6.9.* | 7.0.* )
-            begin "Installing X.Org 6.9/7.0 modules"
+            xserver_version="X.Org 6.9/7.0"
             vboxvideo_src=vboxvideo_drv_70.so
             vboxmouse_src=vboxmouse_drv_70.so
             automouse=""
+            newmouse=""
             ;;
         6.7* | 6.8.* | 4.2.* | 4.3.* )
             # Assume X.Org post-fork or XFree86
-            begin "Installing XFree86 4.2/4.3 and X.Org 6.7/6.8 modules"
-            rm "$modules_dir/drivers/vboxvideo_drv.o" 2>/dev/null
-            rm "$modules_dir/input/vboxmouse_drv.o" 2>/dev/null
-            ln -s "$lib_dir/vboxvideo_drv.o" "$modules_dir/drivers/vboxvideo_drv.o"
-            ln -s "$lib_dir/vboxmouse_drv.o" "$modules_dir/input/vboxmouse_drv.o"
+            xserver_version="XFree86 4.2/4.3 and X.Org 6.7/6.8"
+            driver_ext=.o
+            vboxvideo_src=vboxvideo_drv.o
+            vboxmouse_src=vboxmouse_drv.o
             automouse=""
-            succ_msg
+            newmouse=""
             ;;
         * )
             echo "Warning: unknown version of the X Window System installed.  Not installing"
@@ -421,13 +418,16 @@ setup()
             dox11config=""
             ;;
     esac
-    if [ -n "$vboxvideo_src" -a -n "$vboxmouse_src" ]; then
-        rm "$modules_dir/drivers/vboxvideo_drv.so" 2>/dev/null
-        rm "$modules_dir/input/vboxmouse_drv.so" 2>/dev/null
-        ln -s "$lib_dir/$vboxvideo_src" "$modules_dir/drivers/vboxvideo_drv.so"
-        ln -s "$lib_dir/$vboxmouse_src" "$modules_dir/input/vboxmouse_drv.so" &&
-        succ_msg
-    fi
+    begin "Installing $xserver_version modules"
+    rm "$modules_dir/drivers/vboxvideo_drv$driver_ext" 2>/dev/null
+    rm "$modules_dir/input/vboxmouse_drv$driver_ext" 2>/dev/null
+    case "$vboxvideo_src" in ?*)
+        ln -s "$lib_dir/$vboxvideo_src" "$modules_dir/drivers/vboxvideo_drv$driver_ext";;
+    esac
+    case "$vboxmouse_src" in ?*)
+        ln -s "$lib_dir/$vboxmouse_src" "$modules_dir/input/vboxmouse_drv$driver_ext";;
+    esac
+    succ_msg
 
     if test -n "$dox11config"; then
         begin "Setting up the Window System to use the Guest Additions"
@@ -469,34 +469,6 @@ setup()
                 touch "$nobak"
             fi
         fi
-        # X.Org Server versions starting with 1.5 can do mouse auto-detection,
-        # to make our lives easier and spare us the nasty hacks.
-        test -n "$automouse" &&
-            if [ -d /etc/hal/fdi/policy ]
-            then
-                # Install hal information about the mouse driver so that X.Org
-                # knows to load it.
-                install -o 0 -g 0 -m 0644 "$share_dir/90-vboxguest.fdi" /etc/hal/fdi/policy
-                # Delete the hal cache so that it notices our fdi file
-                rm -r /var/cache/hald/fdi-cache 2> /dev/null
-            fi
-        if test -n "$doxorgconfd"
-        then
-            if test -d /etc/udev/rules.d
-            then
-                install -o 0 -g 0 -m 0644 "$share_dir/70-xorg-vboxmouse.rules" /etc/udev/rules.d
-                # This is normally silent.  I have purposely not redirected
-                # error output as I want to know if something goes wrong,
-                # particularly if the command syntax ever changes.
-                udevadm trigger --action=change --subsystem-match=misc
-            fi
-            test -d /usr/share/X11/xorg.conf.d &&
-                install -o 0 -g 0 -m 0644 "$share_dir/50-vboxmouse.conf" /usr/share/X11/xorg.conf.d
-            test -d /usr/lib/X11/xorg.conf.d &&
-                install -o 0 -g 0 -m 0644 "$share_dir/50-vboxmouse.conf" /usr/lib/X11/xorg.conf.d
-            test -d /etc/X11/xorg.conf.d &&
-                install -o 0 -g 0 -m 0644 "$share_dir/50-vboxmouse.conf" /etc/X11/xorg.conf.d
-        fi
         succ_msg
         test -n "$generated" &&
             cat << EOF
@@ -539,25 +511,17 @@ EOF
     # completely irrelevant on the target system.
     chcon -t unconfined_execmem_exec_t '/usr/bin/VBoxClient' > /dev/null 2>&1
     semanage fcontext -a -t unconfined_execmem_exec_t '/usr/bin/VBoxClient' > /dev/null 2>&1
-    # Install the guest OpenGL drivers
-    if [ -d /usr/lib64/dri ]
-    then
-        rm -f /usr/lib64/dri/vboxvideo_dri.so
-        ln -s $LIB/VBoxOGL.so /usr/lib64/dri/vboxvideo_dri.so
-    elif [ -d /usr/lib/dri ]
-    then
-        rm -f /usr/lib/dri/vboxvideo_dri.so
-        ln -s $LIB/VBoxOGL.so /usr/lib/dri/vboxvideo_dri.so
-    fi
-    if [ -d /usr/lib64/xorg/modules/dri ]
-    then
-        rm -f /usr/lib64/xorg/modules/dri/vboxvideo_dri.so
-        ln -s $LIB/VBoxOGL.so /usr/lib64/xorg/modules/dri/vboxvideo_dri.so
-    elif [ -d /usr/lib/xorg/modules/dri ]
-    then
-        rm -f /usr/lib/xorg/modules/dri/vboxvideo_dri.so
-        ln -s $LIB/VBoxOGL.so /usr/lib/xorg/modules/dri/vboxvideo_dri.so
-    fi
+    # Install the guest OpenGL drivers.  For now we don't support
+    # multi-architecture installations
+    for dir in /usr/lib/dri /usr/lib32/dri /usr/lib64/dri \
+        /usr/lib/xorg/modules/dri /usr/lib32/xorg/modules/dri \
+        /usr/lib64/xorg/modules/dri /usr/lib/i386-linux-gnu/dri \
+        /usr/lib/x86_64-linux-gnu/dri; do
+        if [ -d $dir ]; then
+            rm -f "$dir/vboxvideo_dri.so"
+            ln -s "$LIB/VBoxOGL.so" "$dir/vboxvideo_dri.so"
+        fi
+    done
 
     # And set up VBoxClient to start when the X session does
     install_x11_startup_app "$lib_dir/98vboxadd-xclient" "$share_dir/vboxclient.desktop" VBoxClient VBoxClient-all ||
@@ -638,11 +602,6 @@ EOF
     rm /usr/share/autostart/vboxclient.desktop 2>/dev/null
 
     # Remove other files
-    rm /etc/hal/fdi/policy/90-vboxguest.fdi 2>/dev/null
-    rm /etc/udev/rules.d/70-xorg-vboxmouse.rules 2>/dev/null
-    udevadm trigger --action=change --subsystem-match=misc 2>/dev/null
-    rm /usr/lib/X11/xorg.conf.d/50-vboxmouse.conf 2>/dev/null
-    rm /usr/share/X11/xorg.conf.d/50-vboxmouse.conf 2>/dev/null
     rm /usr/share/xserver-xorg/pci/vboxvideo.ids 2>/dev/null
 }
 
