@@ -1,4 +1,4 @@
-/* $Id: UIMachineLogic.cpp 38431 2011-08-12 11:04:16Z vboxsync $ */
+/* $Id: UIMachineLogic.cpp $ */
 /** @file
  *
  * VBox frontends: Qt GUI ("VirtualBox"):
@@ -45,7 +45,6 @@
 
 /* Global includes */
 #include <iprt/path.h>
-#include <VBox/VMMDev.h>
 
 #ifdef VBOX_WITH_DEBUGGER_GUI
 # include <iprt/ldr.h>
@@ -1454,7 +1453,7 @@ void UIMachineLogic::sltInstallGuestAdditions()
 
     /* Check for the already registered image */
     CVirtualBox vbox = vboxGlobal().virtualBox();
-    QString name = QString("VBoxGuestAdditions_%1.iso").arg(vbox.GetVersion().remove("_OSE"));
+    const QString &name = QString("VBoxGuestAdditions_%1.iso").arg(vboxGlobal().vboxVersionStringNormalized());
 
     CMediumVector vec = vbox.GetDVDImages();
     for (CMediumVector::ConstIterator it = vec.begin(); it != vec.end(); ++ it)
@@ -1470,8 +1469,8 @@ void UIMachineLogic::sltInstallGuestAdditions()
     int result = msgCenter().cannotFindGuestAdditions(QDir::toNativeSeparators(strSrc1), QDir::toNativeSeparators(strSrc2));
     if (result == QIMessageBox::Yes)
     {
-        QString source = QString("http://download.virtualbox.org/virtualbox/%1/").arg(vbox.GetVersion().remove("_OSE")) + name;
-        QString target = QDir(vboxGlobal().virtualBox().GetHomeFolder()).absoluteFilePath(name);
+        const QString &source = QString("http://download.virtualbox.org/virtualbox/%1/").arg(vboxGlobal().vboxVersionStringNormalized()) + name;
+        const QString &target = QDir(vboxGlobal().virtualBox().GetHomeFolder()).absoluteFilePath(name);
 
         UIDownloaderAdditions *pDl = UIDownloaderAdditions::create();
         /* Configure the additions downloader. */
