@@ -1,4 +1,4 @@
-/* $Id: MachineImpl.h 37851 2011-07-08 17:04:03Z vboxsync $ */
+/* $Id: MachineImpl.h $ */
 /** @file
  * VirtualBox COM class implementation
  */
@@ -161,6 +161,7 @@ public:
         // machine settings XML file
         settings::MachineConfigFile *pMachineConfigFile;
         uint32_t            flModifications;
+        bool                m_fAllowStateModification;
 
         BOOL                mAccessible;
         com::ErrorInfo      mAccessError;
@@ -270,8 +271,8 @@ public:
 
         BOOL                 mCPUAttached[SchemaDefs::MaxCPUCount];
 
-        settings::CpuIdLeaf  mCpuIdStdLeafs[10];
-        settings::CpuIdLeaf  mCpuIdExtLeafs[10];
+        settings::CpuIdLeaf  mCpuIdStdLeafs[11];
+        settings::CpuIdLeaf  mCpuIdExtLeafs[11];
 
         DeviceType_T         mBootOrder[SchemaDefs::MaxBootPosition];
 
@@ -646,8 +647,12 @@ public:
      */
     ChipsetType_T getChipsetType() const { return mHWData->mChipsetType; }
 
-    void setModified(uint32_t fl);
-    void setModifiedLock(uint32_t fl);
+    void setModified(uint32_t fl, bool fAllowStateModification = true);
+    void setModifiedLock(uint32_t fl, bool fAllowStateModification = true);
+
+    bool isStateModificationAllowed() const { return mData->m_fAllowStateModification; }
+    void allowStateModification()           { mData->m_fAllowStateModification = true; }
+    void disallowStateModification()        { mData->m_fAllowStateModification = false; }
 
     // callback handlers
     virtual HRESULT onNetworkAdapterChange(INetworkAdapter * /* networkAdapter */, BOOL /* changeAdapter */) { return S_OK; }
