@@ -1090,7 +1090,7 @@ static int vdiCreate(const char *pszFilename, uint64_t cbSize,
                      PVDINTERFACE pVDIfsDisk, PVDINTERFACE pVDIfsImage,
                      PVDINTERFACE pVDIfsOperation, void **ppBackendData)
 {
-    LogFlowFunc(("pszFilename=\"%s\" cbSize=%llu uImageFlags=%#x pszComment=\"%s\" pPCHSGeometry=%#p pLCHSGeometry=%#p Uuid=%RTuuid uOpenFlags=%#x uPercentStart=%u uPercentSpan=%u pVDIfsDisk=%#p pVDIfsImage=%#p pVDIfsOperation=%#p ppBackendData=%#p", pszFilename, cbSize, uImageFlags, pszComment, pPCHSGeometry, pLCHSGeometry, pUuid, uOpenFlags, uPercentStart, uPercentSpan, pVDIfsDisk, pVDIfsImage, pVDIfsOperation, ppBackendData));
+    LogFlowFunc(("pszFilename=\"%s\" cbSize=%llu uImageFlags=%#x pszComment=\"%s\" pPCHSGeometry=%#p pLCHSGeometry=%#p Uuid=%RTuuid uOpenFlags=%#x uPercentStart=%u uPercentSpan=%u pVDIfsDisk=%#p pVDIfsImage=%#p pVDIfsOperation=%#p ppBackendData=%#p\n", pszFilename, cbSize, uImageFlags, pszComment, pPCHSGeometry, pLCHSGeometry, pUuid, uOpenFlags, uPercentStart, uPercentSpan, pVDIfsDisk, pVDIfsImage, pVDIfsOperation, ppBackendData));
     int rc;
     PVDIIMAGEDESC pImage;
 
@@ -1106,7 +1106,7 @@ static int vdiCreate(const char *pszFilename, uint64_t cbSize,
             pfnProgress = pCbProgress->pfnProgress;
         pvUser = pIfProgress->pvUser;
     }
-    
+
     /* Check the image flags. */
     if ((uImageFlags & ~VD_VDI_IMAGE_FLAGS_MASK) != 0)
     {
@@ -1354,7 +1354,8 @@ static int vdiWrite(void *pBackendData, uint64_t uOffset, const void *pvBuf,
                 }
             }
 
-            if (cbToWrite == getImageBlockSize(&pImage->Header))
+            if (   cbToWrite == getImageBlockSize(&pImage->Header)
+                && !(fWrite & VD_WRITE_NO_ALLOC))
             {
                 /* Full block write to previously unallocated block.
                  * Allocate block and write data. */
@@ -2028,11 +2029,6 @@ static void vdiDump(void *pBackendData)
     }
 }
 
-static bool vdiIsAsyncIOSupported(void *pBackendData)
-{
-    return true;
-}
-
 static int vdiAsyncRead(void *pBackendData, uint64_t uOffset, size_t cbToRead,
                         PVDIOCTX pIoCtx, size_t *pcbActuallyRead)
 {
@@ -2740,8 +2736,6 @@ VBOXHDDBACKEND g_VDIBackend =
     NULL,
     /* pfnSetParentFilename */
     NULL,
-    /* pfnIsAsyncIOSupported */
-    vdiIsAsyncIOSupported,
     /* pfnAsyncRead */
     vdiAsyncRead,
     /* pfnAsyncWrite */
