@@ -186,10 +186,8 @@ VMMR3_INT_DECL(int) VMMR3Init(PVM pVM)
     AssertMsgRCReturn(rc, ("Configuration error. Failed to query \"VMM/UsePeriodicPreemptionTimers\", rc=%Rrc\n", rc), rc);
 
     /*
-     * Initialize the VMM sync critical section and semaphores.
+     * Initialize the VMM rendezvous semaphores.
      */
-    rc = RTCritSectInit(&pVM->vmm.s.CritSectSync);
-    AssertRCReturn(rc, rc);
     pVM->vmm.s.pahEvtRendezvousEnterOrdered = (PRTSEMEVENT)MMR3HeapAlloc(pVM, MM_TAG_VMM, sizeof(RTSEMEVENT) * pVM->cCpus);
     if (!pVM->vmm.s.pahEvtRendezvousEnterOrdered)
         return VERR_NO_MEMORY;
@@ -526,7 +524,7 @@ VMMR3_INT_DECL(int) VMMR3InitR0(PVM pVM)
     {
         LogRel(("R0 init failed, rc=%Rra\n", rc));
         if (RT_SUCCESS(rc))
-            rc = VERR_INTERNAL_ERROR;
+            rc = VERR_IPE_UNEXPECTED_INFO_STATUS;
     }
     return rc;
 }
@@ -602,7 +600,7 @@ VMMR3_INT_DECL(int) VMMR3InitRC(PVM pVM)
         {
             VMMR3FatalDump(pVM, pVCpu, rc);
             if (rc >= VINF_EM_FIRST && rc <= VINF_EM_LAST)
-                rc = VERR_INTERNAL_ERROR;
+                rc = VERR_IPE_UNEXPECTED_INFO_STATUS;
         }
         AssertRC(rc);
     }
@@ -736,10 +734,9 @@ VMMR3_INT_DECL(int) VMMR3Term(PVM pVM)
     {
         LogRel(("VMMR3Term: R0 term failed, rc=%Rra. (warning)\n", rc));
         if (RT_SUCCESS(rc))
-            rc = VERR_INTERNAL_ERROR;
+            rc = VERR_IPE_UNEXPECTED_INFO_STATUS;
     }
 
-    RTCritSectDelete(&pVM->vmm.s.CritSectSync);
     for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         RTSemEventDestroy(pVM->vmm.s.pahEvtRendezvousEnterOrdered[i]);
@@ -1364,7 +1361,7 @@ VMMR3_INT_DECL(void) VMMR3SendSipi(PVM pVM, VMCPUID idCpu,  uint32_t uVector)
 {
     AssertReturnVoid(idCpu < pVM->cCpus);
 
-    int rc = VMR3ReqCallNoWaitU(pVM->pUVM, idCpu, (PFNRT)vmmR3SendSipi, 3, pVM, idCpu, uVector);
+    int rc = VMR3ReqCallNoWait(pVM, idCpu, (PFNRT)vmmR3SendSipi, 3, pVM, idCpu, uVector);
     AssertRC(rc);
 }
 
@@ -1378,7 +1375,7 @@ VMMR3_INT_DECL(void) VMMR3SendInitIpi(PVM pVM, VMCPUID idCpu)
 {
     AssertReturnVoid(idCpu < pVM->cCpus);
 
-    int rc = VMR3ReqCallNoWaitU(pVM->pUVM, idCpu, (PFNRT)vmmR3SendInitIpi, 2, pVM, idCpu);
+    int rc = VMR3ReqCallNoWait(pVM, idCpu, (PFNRT)vmmR3SendInitIpi, 2, pVM, idCpu);
     AssertRC(rc);
 }
 
@@ -1392,6 +1389,7 @@ VMMR3_INT_DECL(void) VMMR3SendInitIpi(PVM pVM, VMCPUID idCpu)
  */
 VMMR3DECL(int) VMMR3RegisterPatchMemory(PVM pVM, RTGCPTR pPatchMem, unsigned cbPatchMem)
 {
+    VM_ASSERT_EMT(pVM);
     if (HWACCMIsEnabled(pVM))
         return HWACMMR3EnablePatching(pVM, pPatchMem, cbPatchMem);
 
@@ -1415,62 +1413,6 @@ VMMR3DECL(int) VMMR3DeregisterPatchMemory(PVM pVM, RTGCPTR pPatchMem, unsigned c
 }
 
 
-/**
- * VCPU worker for VMMR3SynchronizeAllVCpus.
- *
- * @param   pVM         The VM to operate on.
- * @param   idCpu       Virtual CPU to perform SIPI on
- * @param   uVector     SIPI vector
- */
-DECLCALLBACK(int) vmmR3SyncVCpu(PVM pVM)
-{
-    /* Block until the job in the caller has finished. */
-    RTCritSectEnter(&pVM->vmm.s.CritSectSync);
-    RTCritSectLeave(&pVM->vmm.s.CritSectSync);
-    return VINF_SUCCESS;
-}
-
-
-/**
- * Atomically execute a callback handler
- * Note: This is very expensive; avoid using it frequently!
- *
- * @param   pVM         The VM to operate on.
- * @param   pfnHandler  Callback handler
- * @param   pvUser      User specified parameter
- *
- * @thread  EMT
- * @todo    Remove this if not used again soon.
- */
-VMMR3DECL(int) VMMR3AtomicExecuteHandler(PVM pVM, PFNATOMICHANDLER pfnHandler, void *pvUser)
-{
-    int    rc;
-    PVMCPU pVCpu = VMMGetCpu(pVM);
-    AssertReturn(pVCpu, VERR_VM_THREAD_NOT_EMT);
-
-    /* Shortcut for the uniprocessor case. */
-    if (pVM->cCpus == 1)
-        return pfnHandler(pVM, pvUser);
-
-    RTCritSectEnter(&pVM->vmm.s.CritSectSync);
-    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
-    {
-        if (idCpu != pVCpu->idCpu)
-        {
-            rc = VMR3ReqCallNoWaitU(pVM->pUVM, idCpu, (PFNRT)vmmR3SyncVCpu, 1, pVM);
-            AssertRC(rc);
-        }
-    }
-    /* Wait until all other VCPUs are waiting for us. */
-    while (RTCritSectGetWaiters(&pVM->vmm.s.CritSectSync) != (int32_t)(pVM->cCpus - 1))
-        RTThreadSleep(1);
-
-    rc = pfnHandler(pVM, pvUser);
-    RTCritSectLeave(&pVM->vmm.s.CritSectSync);
-    return rc;
-}
-
-
 /**
  * Count returns and have the last non-caller EMT wake up the caller.
  *
@@ -1683,8 +1625,12 @@ static int vmmR3EmtRendezvousCommon(PVM pVM, PVMCPU pVCpu, bool fIsCaller,
  */
 VMMR3_INT_DECL(int) VMMR3EmtRendezvousFF(PVM pVM, PVMCPU pVCpu)
 {
-    return vmmR3EmtRendezvousCommon(pVM, pVCpu, false /* fIsCaller */, pVM->vmm.s.fRendezvousFlags,
-                                    pVM->vmm.s.pfnRendezvous, pVM->vmm.s.pvRendezvousUser);
+    Assert(!pVCpu->vmm.s.fInRendezvous);
+    pVCpu->vmm.s.fInRendezvous = true;
+    int rc = vmmR3EmtRendezvousCommon(pVM, pVCpu, false /* fIsCaller */, pVM->vmm.s.fRendezvousFlags,
+                                      pVM->vmm.s.pfnRendezvous, pVM->vmm.s.pvRendezvousUser);
+    pVCpu->vmm.s.fInRendezvous = false;
+    return rc;
 }
 
 
@@ -1727,10 +1673,15 @@ VMMR3DECL(int) VMMR3EmtRendezvous(PVM pVM, uint32_t fFlags, PFNVMMEMTRENDEZVOUS
         rcStrict = VMR3ReqCallWait(pVM, VMCPUID_ANY,
                                    (PFNRT)VMMR3EmtRendezvous, 4, pVM, fFlags, pfnRendezvous, pvUser);
     else if (pVM->cCpus == 1)
+    {
         /*
          * Shortcut for the single EMT case.
          */
+        AssertLogRelReturn(!pVCpu->vmm.s.fInRendezvous, VERR_DEADLOCK);
+        pVCpu->vmm.s.fInRendezvous = true;
         rcStrict = pfnRendezvous(pVM, pVCpu, pvUser);
+        pVCpu->vmm.s.fInRendezvous = false;
+    }
     else
     {
         /*
@@ -1741,6 +1692,8 @@ VMMR3DECL(int) VMMR3EmtRendezvous(PVM pVM, uint32_t fFlags, PFNVMMEMTRENDEZVOUS
         rcStrict = VINF_SUCCESS;
         if (RT_UNLIKELY(!ASMAtomicCmpXchgU32(&pVM->vmm.s.u32RendezvousLock, 0x77778888, 0)))
         {
+            AssertLogRelReturn(!pVCpu->vmm.s.fInRendezvous, VERR_DEADLOCK);
+
             while (!ASMAtomicCmpXchgU32(&pVM->vmm.s.u32RendezvousLock, 0x77778888, 0))
             {
                 if (VM_FF_ISPENDING(pVM, VM_FF_EMT_RENDEZVOUS))
@@ -1756,6 +1709,8 @@ VMMR3DECL(int) VMMR3EmtRendezvous(PVM pVM, uint32_t fFlags, PFNVMMEMTRENDEZVOUS
             }
         }
         Assert(!VM_FF_ISPENDING(pVM, VM_FF_EMT_RENDEZVOUS));
+        Assert(!pVCpu->vmm.s.fInRendezvous);
+        pVCpu->vmm.s.fInRendezvous = true;
 
         /*
          * Clear the slate. This is a semaphore ping-pong orgy. :-)
@@ -1803,6 +1758,7 @@ VMMR3DECL(int) VMMR3EmtRendezvous(PVM pVM, uint32_t fFlags, PFNVMMEMTRENDEZVOUS
         ASMAtomicWriteNullPtr((void * volatile *)&pVM->vmm.s.pfnRendezvous);
 
         ASMAtomicWriteU32(&pVM->vmm.s.u32RendezvousLock, 0);
+        pVCpu->vmm.s.fInRendezvous = false;
 
         /*
          * Merge rcStrict and rcMy.
@@ -1822,6 +1778,25 @@ VMMR3DECL(int) VMMR3EmtRendezvous(PVM pVM, uint32_t fFlags, PFNVMMEMTRENDEZVOUS
 }
 
 
+/**
+ * Disables/enables EMT rendezvous. 
+ *  
+ * This is used to make sure EMT rendezvous does not take place while 
+ * processing a priority request.
+ *  
+ * @returns Old rendezvous-disabled state.
+ * @param   pVCpu           The handle of the calling EMT.
+ * @param   fDisabled       True if disabled, false if enabled.
+ */
+VMMR3_INT_DECL(bool) VMMR3EmtRendezvousSetDisabled(PVMCPU pVCpu, bool fDisabled)
+{
+    VMCPU_ASSERT_EMT(pVCpu);
+    bool fOld = pVCpu->vmm.s.fInRendezvous;
+    pVCpu->vmm.s.fInRendezvous = fDisabled;
+    return fOld;
+}
+
+
 /**
  * Read from the ring 0 jump buffer stack
  *
@@ -1991,7 +1966,7 @@ VMMR3DECL(int) VMMR3CallR0(PVM pVM, uint32_t uOperation, uint64_t u64Arg, PSUPVM
 
     AssertLogRelMsgReturn(rc == VINF_SUCCESS || RT_FAILURE(rc),
                           ("uOperation=%u rc=%Rrc\n", uOperation, rc),
-                          VERR_INTERNAL_ERROR);
+                          VERR_IPE_UNEXPECTED_INFO_STATUS);
     return rc;
 }
 
@@ -2213,7 +2188,7 @@ static int vmmR3ServiceCallRing3Request(PVM pVM, PVMCPU pVCpu)
 
         default:
             AssertMsgFailed(("enmCallRing3Operation=%d\n", pVCpu->vmm.s.enmCallRing3Operation));
-            return VERR_INTERNAL_ERROR;
+            return VERR_VMM_UNKNOWN_RING3_CALL;
     }
 
     pVCpu->vmm.s.enmCallRing3Operation = VMMCALLRING3_INVALID;
